@@ -3,6 +3,7 @@ import os
 import time
 from jinja2 import meta, Environment
 from typing import Optional, Dict 
+from email.message import EmailMessage
 
 from openmailer.smtp_engine import send_email_smtp
 from openmailer.message_builder import build_email_message
@@ -71,18 +72,38 @@ class OpenMailerClient:
         schedule=None,
         priority="normal",
         track_open=False,
-        headers: Optional[Dict[str, str]] = None   # ✅ Add this line
+        batch_id=None,                              # ✅ NEW
+        headers: Optional[Dict[str, str]] = None
     ):
         self._validate_email(to)
         self._validate_attachments(attachments)
         self._validate_template_vars(html_body, context)
-
+    
+        headers = headers or {}
+    
+        # ✅ Inject tracking headers and unsubscribe
+        if context:
+            if "unsubscribe_link" in context:
+                headers["List-Unsubscribe"] = f"<{context['unsubscribe_link']}>"
+                html_body += f'<p><a href="{context["unsubscribe_link"]}">Unsubscribe</a></p>'
+    
+            if "category" in context:
+                headers["X-Category"] = context["category"]
+            if "tags" in context:
+                tags_value = context["tags"]
+                if isinstance(tags_value, list):
+                    tags_value = ",".join(tags_value)
+                headers["X-Tags"] = tags_value
+    
+        if batch_id:
+            headers["X-Batch-ID"] = batch_id  # ✅ NEW
+    
         body = render_template(html_body, context or {})
-
+    
         if track_open:
             tracking_pixel = f'<img src="https://yourdomain.com/track/open/{to}" width="1" height="1" style="display:none">'
             body += tracking_pixel
-
+    
         if schedule:
             self.queue.add({
                 "to": to,
@@ -91,21 +112,21 @@ class OpenMailerClient:
                 "context": context,
                 "attachments": attachments,
                 "priority": priority,
-                "track_open": track_open
+                "track_open": track_open,
+                "batch_id": batch_id  # ✅ NEW
             }, delay=schedule)
-            log_event(to, "scheduled")
+            log_event(to, "scheduled", extra=batch_id)  # ✅ OPTIONAL
             return {"status": "scheduled", "to": to}
-
-        # ✅ Add headers to the message
+    
         message = build_email_message(
             from_email=self.backends[0]["username"],
             to_email=to,
             subject=subject,
             html_body=body,
             attachments=attachments,
-            headers=headers    # ✅ pass headers
+            headers=headers
         )
-
+    
         for backend in self.backends:
             try:
                 if self.dry_run:
@@ -113,9 +134,9 @@ class OpenMailerClient:
                     if report_mode:
                         return {"success": [{"to": to}], "failed": []}
                     return handle_local_send(message, to)
-
+    
                 self.limiters[backend["host"]].wait()
-
+    
                 result = send_email_smtp(
                     smtp_host=backend["host"],
                     smtp_port=backend["port"],
@@ -124,21 +145,21 @@ class OpenMailerClient:
                     use_tls=backend["use_tls"],
                     message=message
                 )
-
+    
                 self.health.record_success(backend["host"])
                 self.analytics.record("sent", backend["host"])
                 log_event(to, "sent", backend["host"])
-
+    
                 if report_mode:
                     return {"success": [{"to": to}], "failed": []}
                 return result
-
+    
             except Exception as e:
                 self.health.record_failure(backend["host"], e)
                 self.analytics.record("failed", backend["host"])
                 log_event(to, "failed", f"{backend['host']}: {str(e)}")
                 last_error = str(e)
-
+    
         self.queue.add({
             "to": to,
             "subject": subject,
@@ -146,13 +167,14 @@ class OpenMailerClient:
             "context": context,
             "attachments": attachments,
             "priority": priority,
-            "track_open": track_open
+            "track_open": track_open,
+            "batch_id": batch_id  # ✅ NEW
         })
-
+    
         if report_mode:
             return {"success": [], "failed": [{"to": to, "error": last_error}]}
         raise Exception(f"❌ All backends failed to send email to {to}: {last_error}")
-
+        
     def send_bulk(self, recipients, subject, html_body, context_fn=None, attachments=None):
         report = {
             "status": "pending",
