@@ -1,9 +1,15 @@
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
 from email.mime.application import MIMEApplication
-from email.message import EmailMessage
 import os
 
+print("✅ Loaded message_builder.py")
+
+try:
+    EmailMessage
+except NameError:
+    from email.message import EmailMessage
+
 def build_email_message(from_email, to_email, subject, html_body, attachments=None, headers=None):
     msg = EmailMessage()
     msg["From"] = from_email
